@@ -409,25 +409,49 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                             'namespace': target_ref['namespace']}}
         project_id = self._drv_pod_project.get_project(pod)
         subnets_map = self._drv_pod_subnets.get_subnets(pod, project_id)
-        # FIXME(ivc): potentially unsafe [0] index
-        return [subnet_id for subnet_id, network in subnets_map.items()
-                for subnet in network.subnets.objects
-                if ip in subnet.cidr][0]
+        subnet_ids = [subnet_id for subnet_id, network in subnets_map.items()
+                      for subnet in network.subnets.objects
+                      if ip in subnet.cidr]
+        if subnet_ids:
+            return subnet_ids[0]
+        else:
+            # NOTE(ltomasbo): We are assuming that if ip is not on the
+            # pod subnet is because the member is using hostnetworking. In
+            # this worker_nodes_subnet will be used
+            return config.CONF.pod_vif_nested.worker_nodes_subnet
+
+    def _get_port_in_pool(self, pool, lbaas_state, lbaas_spec):
+        for l in lbaas_state.listeners:
+            if l.id != pool.listener_id:
+                continue
+            for port in lbaas_spec.ports:
+                if l.port == port.port and l.protocol == port.protocol:
+                    return port
+        return None
 
     def _remove_unused_members(self, endpoints, lbaas_state, lbaas_spec):
-        spec_port_names = {p.name for p in lbaas_spec.ports}
-        current_targets = {(a['ip'], p['port'])
+        spec_ports = {}
+        for pool in lbaas_state.pools:
+            port = self._get_port_in_pool(pool, lbaas_state, lbaas_spec)
+            if port:
+                spec_ports[port.name] = pool.id
+
+        current_targets = {(a['ip'], p['port'],
+                            spec_ports.get(p.get('name')))
                            for s in endpoints['subsets']
                            for a in s['addresses']
                            for p in s['ports']
-                           if p.get('name') in spec_port_names}
+                           if p.get('name') in spec_ports}
+
         removed_ids = set()
         for member in lbaas_state.members:
-            if (str(member.ip), member.port) in current_targets:
+            if ((str(member.ip), member.port, member.pool_id) in
+                    current_targets):
                 continue
             self._drv_lbaas.release_member(lbaas_state.loadbalancer,
                                            member)
             removed_ids.add(member.id)
+
         if removed_ids:
             lbaas_state.members = [m for m in lbaas_state.members
                                    if m.id not in removed_ids]
