@@ -28,6 +28,7 @@ from kuryr_kubernetes.controller.drivers import nested_vlan_vif
 from kuryr_kubernetes.controller.drivers import neutron_vif
 from kuryr_kubernetes.controller.drivers import vif_pool
 from kuryr_kubernetes import exceptions
+from kuryr_kubernetes.objects import vif
 from kuryr_kubernetes import os_vif_util as ovu
 from kuryr_kubernetes.tests import base as test_base
 from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
@@ -61,6 +62,10 @@ def get_pod_obj():
         }}
 
 
+def get_pod_name(pod):
+    return "%(namespace)s/%(name)s" % pod['metadata']
+
+
 def get_port_obj(port_id=None, device_owner=None, ip_address=None):
     port_obj = {
         'allowed_address_pairs': [],
@@ -260,6 +265,7 @@ class BaseVIFPool(test_base.TestCase):
         cls = vif_pool.BaseVIFPool
         m_driver = mock.MagicMock(spec=cls)
         m_driver._recyclable_ports = {}
+        m_driver._existing_vifs = {}
 
         pod = get_pod_obj()
         project_id = mock.sentinel.project_id
@@ -282,15 +288,11 @@ class BaseVIFPool(test_base.TestCase):
         kubernetes = self.useFixture(k_fix.MockK8sClient()).client
         pod = get_pod_obj()
         port_id = uuidutils.generate_uuid()
-        versioned_object = jsonutils.dumps({
-            'versioned_object.data': {
-                'active': True,
-                'address': 'fa:16:3e:ef:e6:9f',
-                'id': port_id
-            }})
+        pod_vif = osv_vif.VIFBase(id=port_id)
+        pod_state = vif.PodState(default_vif=pod_vif)
 
         pod['metadata']['annotations'][constants.K8S_ANNOTATION_VIF] = (
-            versioned_object)
+            jsonutils.dumps(pod_state.obj_to_primitive()))
         items = [pod]
         kubernetes.get.return_value = {'items': items}
 
@@ -328,8 +330,9 @@ class BaseVIFPool(test_base.TestCase):
 @ddt.ddt
 class NeutronVIFPool(test_base.TestCase):
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_port_name')
     @mock.patch('eventlet.spawn')
-    def test__get_port_from_pool(self, m_eventlet):
+    def test__get_port_from_pool(self, m_eventlet, m_get_port_name):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -344,6 +347,7 @@ class NeutronVIFPool(test_base.TestCase):
         m_driver._available_ports_pools = {
             pool_key: collections.deque([port_id])}
         m_driver._existing_vifs = {port_id: port}
+        m_get_port_name.return_value = get_pod_name(pod)
 
         oslo_cfg.CONF.set_override('ports_pool_min',
                                    5,
@@ -364,14 +368,16 @@ class NeutronVIFPool(test_base.TestCase):
             port_id,
             {
                 "port": {
-                    'name': pod['metadata']['name'],
+                    'name': get_pod_name(pod),
                     'device_id': pod['metadata']['uid']
                 }
             })
         m_eventlet.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_port_name')
     @mock.patch('eventlet.spawn')
-    def test__get_port_from_pool_pool_populate(self, m_eventlet):
+    def test__get_port_from_pool_pool_populate(self, m_eventlet,
+                                               m_get_port_name):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -386,6 +392,7 @@ class NeutronVIFPool(test_base.TestCase):
         m_driver._available_ports_pools = {
             pool_key: collections.deque([port_id])}
         m_driver._existing_vifs = {port_id: port}
+        m_get_port_name.return_value = get_pod_name(pod)
 
         oslo_cfg.CONF.set_override('ports_pool_min',
                                    5,
@@ -403,7 +410,7 @@ class NeutronVIFPool(test_base.TestCase):
             port_id,
             {
                 "port": {
-                    'name': pod['metadata']['name'],
+                    'name': get_pod_name(pod),
                     'device_id': pod['metadata']['uid']
                 }
             })
@@ -425,9 +432,8 @@ class NeutronVIFPool(test_base.TestCase):
 
         neutron.update_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
     @ddt.data((0), (10))
-    def test__return_ports_to_pool(self, max_pool, m_sleep):
+    def test__trigger_return_to_pool(self, max_pool):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -448,7 +454,7 @@ class NeutronVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_called_once_with(
             port_id,
@@ -461,9 +467,8 @@ class NeutronVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
     @ddt.data((0), (10))
-    def test__return_ports_to_pool_no_update(self, max_pool, m_sleep):
+    def test__trigger_return_to_pool_no_update(self, max_pool):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -484,13 +489,12 @@ class NeutronVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group']}]
         m_driver._get_pool_size.return_value = pool_length
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_delete_port(self, m_sleep):
+    def test__trigger_return_to_pool_delete_port(self):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -510,13 +514,12 @@ class NeutronVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_called_once_with(port_id)
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_update_exception(self, m_sleep):
+    def test__trigger_return_to_pool_update_exception(self):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -541,7 +544,7 @@ class NeutronVIFPool(test_base.TestCase):
         m_driver._get_pool_size.return_value = pool_length
         neutron.update_port.side_effect = n_exc.NeutronClientException
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_called_once_with(
             port_id,
@@ -554,8 +557,7 @@ class NeutronVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_delete_exception(self, m_sleep):
+    def test__trigger_return_to_pool_delete_exception(self):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -576,13 +578,12 @@ class NeutronVIFPool(test_base.TestCase):
         m_driver._get_pool_size.return_value = pool_length
         neutron.delete_port.side_effect = n_exc.PortNotFoundClient
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_called_once_with(port_id)
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_delete_key_error(self, m_sleep):
+    def test__trigger_return_to_pool_delete_key_error(self):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -601,14 +602,13 @@ class NeutronVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_not_called()
 
     @mock.patch('kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif')
-    @mock.patch('kuryr_kubernetes.controller.drivers.default_subnet.'
-                '_get_subnet')
+    @mock.patch('kuryr_kubernetes.utils.get_subnet')
     def test__recover_precreated_ports(self, m_get_subnet, m_to_osvif):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
@@ -654,8 +654,7 @@ class NeutronVIFPool(test_base.TestCase):
         self.assertEqual(m_driver._available_ports_pools[pool_key], [port_id])
 
     @mock.patch('kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif')
-    @mock.patch('kuryr_kubernetes.controller.drivers.default_subnet.'
-                '_get_subnet')
+    @mock.patch('kuryr_kubernetes.utils.get_subnet')
     def test__recover_precreated_ports_empty(self, m_get_subnet, m_to_osvif):
         cls = vif_pool.NeutronVIFPool
         m_driver = mock.MagicMock(spec=cls)
@@ -673,6 +672,47 @@ class NeutronVIFPool(test_base.TestCase):
         m_get_subnet.assert_not_called()
         m_to_osvif.assert_not_called()
 
+    def test_delete_network_pools(self):
+        cls = vif_pool.NeutronVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        net_id = mock.sentinel.net_id
+        pool_key = ('node_ip', 'project_id', tuple(['security_group']))
+        port_id = uuidutils.generate_uuid()
+        m_driver._available_ports_pools = {pool_key: [port_id]}
+        m_driver._existing_vifs = {port_id: mock.sentinel.vif}
+
+        m_driver._get_pool_key_net.return_value = net_id
+
+        cls.delete_network_pools(m_driver, net_id)
+
+        m_driver._trigger_return_to_pool.assert_called_once()
+        m_driver._get_pool_key_net.assert_called_once()
+        neutron.delete_port.assert_called_once_with(port_id)
+
+    def test_delete_network_pools_missing_port_id(self):
+        cls = vif_pool.NeutronVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        net_id = mock.sentinel.net_id
+        pool_key = ('node_ip', 'project_id', tuple(['security_group']))
+        port_id = uuidutils.generate_uuid()
+        m_driver._available_ports_pools = {pool_key: [port_id]}
+        m_driver._existing_vifs = {}
+        neutron.delete_port.side_effect = n_exc.PortNotFoundClient
+
+        m_driver._get_pool_key_net.return_value = net_id
+
+        cls.delete_network_pools(m_driver, net_id)
+
+        m_driver._trigger_return_to_pool.assert_called_once()
+        m_driver._get_pool_key_net.assert_called_once()
+        neutron.delete_port.assert_called_once_with(port_id)
+
 
 @ddt.ddt
 class NestedVIFPool(test_base.TestCase):
@@ -712,8 +752,9 @@ class NestedVIFPool(test_base.TestCase):
                 'subports': trunk_obj['sub_ports']}
         return parent_ports
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_port_name')
     @mock.patch('eventlet.spawn')
-    def test__get_port_from_pool(self, m_eventlet):
+    def test__get_port_from_pool(self, m_eventlet, m_get_port_name):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -728,6 +769,7 @@ class NestedVIFPool(test_base.TestCase):
         m_driver._available_ports_pools = {
             pool_key: collections.deque([port_id])}
         m_driver._existing_vifs = {port_id: port}
+        m_get_port_name.return_value = get_pod_name(pod)
 
         oslo_cfg.CONF.set_override('ports_pool_min',
                                    5,
@@ -745,13 +787,15 @@ class NestedVIFPool(test_base.TestCase):
             port_id,
             {
                 "port": {
-                    'name': pod['metadata']['name'],
+                    'name': get_pod_name(pod),
                 }
             })
         m_eventlet.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_port_name')
     @mock.patch('eventlet.spawn')
-    def test__get_port_from_pool_pool_populate(self, m_eventlet):
+    def test__get_port_from_pool_pool_populate(self, m_eventlet,
+                                               m_get_port_name):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -766,6 +810,7 @@ class NestedVIFPool(test_base.TestCase):
         m_driver._available_ports_pools = {
             pool_key: collections.deque([port_id])}
         m_driver._existing_vifs = {port_id: port}
+        m_get_port_name.return_value = get_pod_name(pod)
 
         oslo_cfg.CONF.set_override('ports_pool_min',
                                    5,
@@ -783,7 +828,7 @@ class NestedVIFPool(test_base.TestCase):
             port_id,
             {
                 "port": {
-                    'name': pod['metadata']['name'],
+                    'name': get_pod_name(pod),
                 }
             })
         m_eventlet.assert_called_once()
@@ -804,9 +849,8 @@ class NestedVIFPool(test_base.TestCase):
 
         neutron.update_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
     @ddt.data((0), (10))
-    def test__return_ports_to_pool(self, max_pool, m_sleep):
+    def test__trigger_return_to_pool(self, max_pool):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -827,7 +871,7 @@ class NestedVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_called_once_with(
             port_id,
@@ -839,9 +883,8 @@ class NestedVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
     @ddt.data((0), (10))
-    def test__return_ports_to_pool_no_update(self, max_pool, m_sleep):
+    def test__trigger_return_to_pool_no_update(self, max_pool):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -862,13 +905,12 @@ class NestedVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group']}]
         m_driver._get_pool_size.return_value = pool_length
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_delete_port(self, m_sleep):
+    def test__trigger_return_to_pool_delete_port(self):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -881,7 +923,6 @@ class NestedVIFPool(test_base.TestCase):
         pool_length = 10
         vif = mock.MagicMock()
         vif.vlan_id = mock.sentinel.vlan_id
-        p_port = mock.sentinel.p_port
         trunk_id = uuidutils.generate_uuid()
 
         m_driver._recyclable_ports = {port_id: pool_key}
@@ -893,22 +934,19 @@ class NestedVIFPool(test_base.TestCase):
         m_driver._get_ports_by_attrs.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
+        m_driver._get_trunk_id.return_value = trunk_id
         m_driver._known_trunk_ids = {}
-        m_driver._drv_vif._get_parent_port_by_host_ip.return_value = p_port
-        m_driver._drv_vif._get_trunk_id.return_value = trunk_id
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
         neutron.delete_port.assert_called_once_with(port_id)
-        m_driver._drv_vif._get_parent_port_by_host_ip.assert_called_once()
-        m_driver._drv_vif._get_trunk_id.assert_called_once_with(p_port)
+        m_driver._get_trunk_id.assert_called_once()
         m_driver._drv_vif._remove_subport.assert_called_once_with(neutron,
                                                                   trunk_id,
                                                                   port_id)
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_update_exception(self, m_sleep):
+    def test__trigger_return_to_pool_update_exception(self):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -930,7 +968,7 @@ class NestedVIFPool(test_base.TestCase):
         m_driver._get_pool_size.return_value = pool_length
         neutron.update_port.side_effect = n_exc.NeutronClientException
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_called_once_with(
             port_id,
@@ -942,8 +980,7 @@ class NestedVIFPool(test_base.TestCase):
             })
         neutron.delete_port.assert_not_called()
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_delete_exception(self, m_sleep):
+    def test__trigger_return_to_pool_delete_exception(self):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -956,7 +993,6 @@ class NestedVIFPool(test_base.TestCase):
         pool_length = 10
         vif = mock.MagicMock()
         vif.vlan_id = mock.sentinel.vlan_id
-        p_port = mock.sentinel.p_port
         trunk_id = uuidutils.generate_uuid()
 
         m_driver._recyclable_ports = {port_id: pool_key}
@@ -968,23 +1004,20 @@ class NestedVIFPool(test_base.TestCase):
         m_driver._get_ports_by_attrs.return_value = [
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
+        m_driver._get_trunk_id.return_value = trunk_id
         neutron.delete_port.side_effect = n_exc.PortNotFoundClient
         m_driver._known_trunk_ids = {}
-        m_driver._drv_vif._get_parent_port_by_host_ip.return_value = p_port
-        m_driver._drv_vif._get_trunk_id.return_value = trunk_id
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
-        m_driver._drv_vif._get_parent_port_by_host_ip.assert_called_once()
-        m_driver._drv_vif._get_trunk_id.assert_called_once_with(p_port)
+        m_driver._get_trunk_id.assert_called_once()
         m_driver._drv_vif._remove_subport.assert_called_once_with(neutron,
                                                                   trunk_id,
                                                                   port_id)
         neutron.delete_port.assert_called_once_with(port_id)
 
-    @mock.patch('eventlet.sleep', side_effect=SystemExit)
-    def test__return_ports_to_pool_delete_key_error(self, m_sleep):
+    def test__trigger_return_to_pool_delete_key_error(self):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
@@ -995,7 +1028,6 @@ class NestedVIFPool(test_base.TestCase):
         pool_key = ('node_ip', 'project_id', tuple(['security_group']))
         port_id = uuidutils.generate_uuid()
         pool_length = 10
-        p_port = mock.sentinel.p_port
         trunk_id = uuidutils.generate_uuid()
 
         m_driver._recyclable_ports = {port_id: pool_key}
@@ -1008,14 +1040,12 @@ class NestedVIFPool(test_base.TestCase):
             {'id': port_id, 'security_groups': ['security_group_modified']}]
         m_driver._get_pool_size.return_value = pool_length
         m_driver._known_trunk_ids = {}
-        m_driver._drv_vif._get_parent_port_by_host_ip.return_value = p_port
-        m_driver._drv_vif._get_trunk_id.return_value = trunk_id
+        m_driver._get_trunk_id.return_value = trunk_id
 
-        self.assertRaises(SystemExit, cls._return_ports_to_pool, m_driver)
+        cls._trigger_return_to_pool(m_driver)
 
         neutron.update_port.assert_not_called()
-        m_driver._drv_vif._get_parent_port_by_host_ip.assert_called_once()
-        m_driver._drv_vif._get_trunk_id.assert_called_once_with(p_port)
+        m_driver._get_trunk_id.assert_called_once()
         m_driver._drv_vif._remove_subport.assert_called_once_with(neutron,
                                                                   trunk_id,
                                                                   port_id)
@@ -1035,8 +1065,7 @@ class NestedVIFPool(test_base.TestCase):
         self.assertEqual(ip_address, cls._get_parent_port_ip(m_driver,
                                                              port_id))
 
-    @mock.patch('kuryr_kubernetes.controller.drivers.default_subnet.'
-                '_get_subnet')
+    @mock.patch('kuryr_kubernetes.utils.get_subnet')
     def test__get_trunk_info(self, m_get_subnet):
         cls = vif_pool.NestedVIFPool
         m_driver = mock.MagicMock(spec=cls)
@@ -1340,3 +1369,105 @@ class NestedVIFPool(test_base.TestCase):
         self.assertEqual(m_driver._existing_vifs, {})
         self.assertEqual(m_driver._available_ports_pools, {})
         neutron.delete_port.assert_not_called()
+
+    def test_delete_network_pools(self):
+        cls = vif_pool.NestedVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        cls_vif_driver = nested_vlan_vif.NestedVlanPodVIFDriver
+        vif_driver = mock.MagicMock(spec=cls_vif_driver)
+        m_driver._drv_vif = vif_driver
+
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        net_id = mock.sentinel.net_id
+        pool_key = ('node_ip', 'project_id', tuple(['security_group']))
+        port_id = uuidutils.generate_uuid()
+        trunk_id = uuidutils.generate_uuid()
+        vif = mock.MagicMock()
+        vlan_id = mock.sentinel.vlan_id
+        vif.vlan_id = vlan_id
+        m_driver._available_ports_pools = {pool_key: [port_id]}
+        m_driver._existing_vifs = {port_id: vif}
+
+        m_driver._get_trunk_id.return_value = trunk_id
+        m_driver._get_pool_key_net.return_value = net_id
+
+        cls.delete_network_pools(m_driver, net_id)
+
+        m_driver._trigger_return_to_pool.assert_called_once()
+        m_driver._get_pool_key_net.assert_called_once()
+        m_driver._get_trunk_id.assert_called_once_with(neutron, pool_key)
+        m_driver._drv_vif._remove_subports.assert_called_once_with(neutron,
+                                                                   trunk_id,
+                                                                   [port_id])
+        m_driver._drv_vif._release_vlan_id.assert_called_once_with(vlan_id)
+        neutron.delete_port.assert_called_once_with(port_id)
+
+    def test_delete_network_pools_exception(self):
+        cls = vif_pool.NestedVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        cls_vif_driver = nested_vlan_vif.NestedVlanPodVIFDriver
+        vif_driver = mock.MagicMock(spec=cls_vif_driver)
+        m_driver._drv_vif = vif_driver
+
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        net_id = mock.sentinel.net_id
+        pool_key = ('node_ip', 'project_id', tuple(['security_group']))
+        port_id = uuidutils.generate_uuid()
+        trunk_id = uuidutils.generate_uuid()
+        vif = mock.MagicMock()
+        vlan_id = mock.sentinel.vlan_id
+        vif.vlan_id = vlan_id
+        m_driver._available_ports_pools = {pool_key: [port_id]}
+        m_driver._existing_vifs = {port_id: vif}
+
+        m_driver._get_trunk_id.return_value = trunk_id
+        m_driver._get_pool_key_net.return_value = net_id
+        m_driver._drv_vif._remove_subports.side_effect = (
+            n_exc.NeutronClientException)
+
+        cls.delete_network_pools(m_driver, net_id)
+
+        m_driver._trigger_return_to_pool.assert_called_once()
+        m_driver._get_pool_key_net.assert_called_once()
+        m_driver._get_trunk_id.assert_called_once_with(neutron, pool_key)
+        m_driver._drv_vif._remove_subports.assert_called_once_with(neutron,
+                                                                   trunk_id,
+                                                                   [port_id])
+        m_driver._drv_vif._release_vlan_id.assert_not_called()
+        neutron.delete_port.assert_not_called()
+
+    def test_delete_network_pools_missing_port(self):
+        cls = vif_pool.NestedVIFPool
+        m_driver = mock.MagicMock(spec=cls)
+        cls_vif_driver = nested_vlan_vif.NestedVlanPodVIFDriver
+        vif_driver = mock.MagicMock(spec=cls_vif_driver)
+        m_driver._drv_vif = vif_driver
+
+        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+
+        net_id = mock.sentinel.net_id
+        pool_key = ('node_ip', 'project_id', tuple(['security_group']))
+        port_id = uuidutils.generate_uuid()
+        trunk_id = uuidutils.generate_uuid()
+        vif = mock.MagicMock()
+        vlan_id = mock.sentinel.vlan_id
+        vif.vlan_id = vlan_id
+        m_driver._available_ports_pools = {pool_key: [port_id]}
+        m_driver._existing_vifs = {}
+
+        m_driver._get_trunk_id.return_value = trunk_id
+        m_driver._get_pool_key_net.return_value = net_id
+        neutron.delete_port.side_effect = n_exc.PortNotFoundClient
+
+        cls.delete_network_pools(m_driver, net_id)
+
+        m_driver._trigger_return_to_pool.assert_called_once()
+        m_driver._get_pool_key_net.assert_called_once()
+        m_driver._get_trunk_id.assert_called_once_with(neutron, pool_key)
+        m_driver._drv_vif._remove_subports.assert_called_once_with(neutron,
+                                                                   trunk_id,
+                                                                   [port_id])
+        m_driver._drv_vif._release_vlan_id.assert_not_called()
+        neutron.delete_port.assert_called_once_with(port_id)
